@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -48,7 +48,6 @@ typedef struct
 	bool		has_insertinto_or_locking_clause;	/* True if it has SELECT
 													 * INTO or FOR
 													 * SHARE/UPDATE */
-	bool		row_security;	/* true if row security enabled */
 	int			num_oids;		/* number of oids */
 	int			table_oids[POOL_MAX_SELECT_OIDS];	/* table oids */
 	char		table_names[POOL_MAX_SELECT_OIDS][NAMEDATALEN];	/* table names */
@@ -62,7 +61,6 @@ extern bool pool_has_temp_table(Node *node);
 extern void discard_temp_table_relcache(void);
 extern bool pool_has_unlogged_table(Node *node);
 extern bool pool_has_view(Node *node);
-extern bool pool_has_row_security(Node *node);
 extern bool pool_has_insertinto_or_locking_clause(Node *node);
 extern bool pool_has_pgpool_regclass(void);
 extern bool pool_has_to_regclass(void);
@@ -71,6 +69,7 @@ extern int	pool_table_name_to_oid(char *table_name);
 extern int	pool_extract_table_oids_from_select_stmt(Node *node, SelectContext * ctx);
 extern RangeVar *makeRangeVarFromNameList(List *names);
 extern char *make_table_name_from_rangevar(RangeVar *rangevar);
+extern char *make_function_name_from_funccall(FuncCall *fcall);
 extern int	pattern_compare(char *str, const int type, const char *param_name);
 extern bool is_unlogged_table(char *table_name);
 extern bool is_view(char *table_name);
